local KeyMap = require("KeyMap")

buttons = {}

function love.load()

	KeyMap:addMapping({"up","down","left","right","fire"})
	KeyMap:bind( "up" , {"key", "w", "up"} )
	KeyMap:bind( "up" , {"joy" , "dpup" , "up" } )
	KeyMap:bind( "down" , {"key", "s", "down"} )
	KeyMap:bind( "down" , {"joy" , "dpdown" , "down" } )
	KeyMap:bind( "left" , {"key", "a", "left"} )
	KeyMap:bind( "left" , {"joy" , "dpleft" , "left" } )
	KeyMap:bind( "right" , {"key", "d", "right"} )
	KeyMap:bind( "right" , {"joy" , "dpright" , "right" } )
	KeyMap:bind( "fire" , {"key", " "} )
	KeyMap:bind( "fire" , {"joy" , "a" } )
	screenWidth = love.graphics.getWidth()
	screenHeight = love.graphics.getHeight()
	
	local num = 1
	for i , k in pairs( KeyMap:getMapping() ) do
	
		table.insert( buttons , { x = screenWidth - 100, y = num * 20 , w = 90 , h = 15 , id = i })
		
		num = num + 1
	
	end
	
end

function love.update(dt)

	KeyMap:update(dt)
	
end

function love.mousereleased( button )

	mx = love.mouse.getX()
	my = love.mouse.getY()
	
	for i , b in pairs( buttons ) do
		if CheckCollision( mx , my , b ) then
			
			if not KeyMap:isBinding() then
				KeyMap:addBind(b.id)
			else
				KeyMap:unbind(b.id)
			end
			
		end
	end

end

function love.draw()
	love.graphics.setColor( 255 , 255 , 255 , 255 )
	
	love.graphics.print("Click \"BIND\" and press a button to bind." , 10 , 200 )
	love.graphics.print("Click \"BIND\" and click again to unbind all." , 10 , 220 )
	love.graphics.print("Click \"BIND\" and press escape to cancel binding." , 10 , 240 )
	
	
	local num = 1
	for i , k in pairs( KeyMap:getMapping() ) do
	
		if KeyMap:isDown( i ) then
			love.graphics.setColor( 255 , 255 , 0 , 255 )
			love.graphics.rectangle( "fill" , 5 , 2 + num * 20 , 10 , 10)
			love.graphics.setColor( 255 , 255 , 255 , 255)
		end
		
		if KeyMap:isPushed( i ) then
			love.graphics.setColor( 255 , 255 , 0 , 255 )
			love.graphics.rectangle( "fill" , 15 , 2 + num * 20 , 10 , 10)
			love.graphics.setColor( 255 , 255 , 255 , 255)
		end
		
		if KeyMap:isHeld( i ) then
			love.graphics.setColor( 255 , 255 , 0 , 255 )
			love.graphics.rectangle( "fill" , 25 , 2 + num * 20 , 10 , 10)
			love.graphics.setColor( 255 , 255 , 255 , 255)
		end
	
		love.graphics.print( tostring(i) ,50 , num * 20 )
		
		for j , m in pairs( k.keys ) do
			
			if m == " " then
				love.graphics.print( "space" , 100 + 50 * j , num * 20 )
			else
				love.graphics.print( m , 100 + 50 * j , num * 20 )
			end
			
		end
		
		for j , m in pairs( k.joy ) do
			
			love.graphics.print( m , 300 + 55 * j , num * 20 )
			
		end
		
		love.graphics.rectangle( "fill" , buttons[num].x , buttons[num].y , buttons[num].w , buttons[num].h)
		
		if not KeyMap:isBinding() then
			love.graphics.setColor( 0 , 0 , 0 , 255 )
			love.graphics.print("BIND" , buttons[num].x + 30 , buttons[num].y )
			love.graphics.setColor( 255 , 255 , 255 , 255 )
		end
		
		
		
		num = num + 1
		
	end

end

function CheckCollision(x,y,box)
  return x < box.x+box.w and x > box.x and y < box.y + box.h and  y > box.y
end